(*

   FngMouseNtfy.Pas
   FnugrMouseNotify - A Simple system-wide mouse message hook component
   Version 1.1.0.0

   Copyright (C) 1997-98 Gleb Yourchenko

   Contact:
   Gleb Yourchenko
   eip__@hotmail.com
   Please include 'FnugryMouseNotify' in the subject string.

   Revision History:

   1.1.0.0          22,apr,1998 Gleb Yourchenko
                    The component was modified to allow
                    mouse event handler to discard mouse
                    message. A new "fDiscard" parameter has
                    been added to the MouseNotifyEvent.

*)


unit FngMouseNtfy;

interface
uses
   Windows, SysUtils, Classes, Messages;

type

  EMouseNotifyError = class(Exception);

  TMouseNotifyEvent = procedure(Sender :TObject; Msg :Integer;
    lpInfo :PMOUSEHOOKSTRUCT; var fDiscard :Boolean) of object;



  TFnugryMouseNotify = class(TComponent)
  private
     FHandle  :HWND;
     FOnMouseMessage    :TMouseNotifyEvent;
     FOnChange          :TNotifyEvent;
     procedure InstallHook;
     procedure ReleaseHook;
     function GetEnabled :Boolean;
     procedure SetEnabled(Value :Boolean);
     procedure HookWindowProc(Var M :TMessage);
  protected
     procedure MouseMessage(Msg :Integer; lpInfo :PMOUSEHOOKSTRUCT;
        var fDiscard :Boolean); virtual;
     procedure Change; virtual;
  public
     destructor Destroy; override;
  published
     property Enabled :Boolean
        read GetEnabled write SetEnabled;
     property OnMouseMessage :TMouseNotifyEvent
        read FOnMouseMessage write FOnMouseMessage;
     property OnChange :TNotifyEvent
        read FOnChange write FOnChange;
  end;

procedure Register;

implementation
uses
   fngmhlib, forms;


{$Resource FngMouseNtfy.Res}
{$Include FngMouseNtfy.Inc}

procedure Register;
begin
   RegisterComponents('Fnugry Tools', [TFnugryMouseNotify]);
end;

const

   WM_MOUSE_NOTIFY      = WM_USER + 36;

procedure TFnugryMouseNotify.InstallHook;
begin
   assert(FHandle = 0);
   FHandle := AllocateHWnd(HookWindowProc);
   try
      if ( fngmhlib.InstallMouseHook(FHandle, WM_MOUSE_NOTIFY) = -1 ) then
         raise EMouseNotifyError.CreateRes(msg_err_installhook);
   except
      DeallocateHWnd(FHandle);
      FHandle := 0;
      raise;
   end;
end;

procedure TFnugryMouseNotify.ReleaseHook;
begin
   if FHandle <> 0 then
      begin
         fngmhlib.UninstallMouseHook(FHandle);
         DeallocateHWnd(FHandle);
         FHandle := 0;
      end;
end;


procedure TFnugryMouseNotify.SetEnabled(Value :Boolean);
begin
   if Enabled <> Value then
      begin
         if Value then
            InstallHook
         else
            ReleaseHook;
         Change;
      end;
end;


procedure TFnugryMouseNotify.MouseMessage(Msg :Integer; lpInfo :PMOUSEHOOKSTRUCT;
   var fDiscard :Boolean);
begin
  if assigned(FOnMouseMessage) then
     FOnMouseMessage(Self, Msg, lpInfo, fDiscard);
end;

function TFnugryMouseNotify.GetEnabled :Boolean;
begin
  result := FHandle <> 0;
end;

destructor TFnugryMouseNotify.Destroy;
begin
   Enabled := false;
   inherited Destroy;
end;


procedure TFnugryMouseNotify.HookWindowProc(Var M :TMessage);
var
   fDiscard :Boolean;
begin
   m.result := 0;
   if M.msg = WM_MOUSE_NOTIFY then
      begin
         fDiscard := false;
         MouseMessage(M.wParam, Pointer(M.lParam), fDiscard);
         if fDiscard then m.result := 1;
      end;
end;

procedure TFnugryMouseNotify.Change;
begin
  if assigned(FOnChange) then
     FOnChange(Self);
end;

end.


